page 48,132
title	COM-AND script machine language subroutine
;-------------------------------------------------------------------
;	Author:      R. McGinnis; Chicago IL
;
;	These routines provide a replacement for COM-AND's critical
;	error handler, allow its replacement (in some cases).
;
;	Source: CRITERR.ASM	Source for this module
;	Object: CRITERR.OBJ	Object derived from assembly of .Asm
;	Exec:	CRITERR.EXE	Executable derived from LINK of .OBJ
;
;	Note: you'll need entry points for the script SCALL statement
;	Note: no stack segment is required
;-------------------------------------------------------------------

code	segment byte public 'code'
	ASSUME	CS:Code

	jmp	near ptr Open		; A jump table makes it easy
	jmp	near ptr GetFlag	; .. to SCALL a routine
	jmp	near ptr Close		; ...

;------ Data -------------------------------------------------------
;	This segment serves as a small storage area for this procedure.
;
;-------------------------------------------------------------------

data	segment byte public 'data'
	Vector_Used	dw	0	; Flag to indicate vector used
	Saved_Vector	dw	0,0	; Saved original vector value
data	ends

	ASSUME	DS:Data

page
;------ Open -------------------------------------------------------
;	This procedure 'opens' the module.  It takes the DOS critical
;	error vector (if not already taken), and clears a flag used
;	determine if the handler has been invoked.
;
;	Passed:
;	   Nothing
;	Returned:
;	   One parameter, 0 -> DOS version does not allow use
;
;	No registers need be preserved...
;-------------------------------------------------------------------

Open	proc	far			; MUST be far return !!
	mov	BP,SP			; Save current stack ptr
	push	BP			; Save value on stack
;
;	Read the DOS version and test if > 3.0
;
	mov	ah,30H			; Ftn -> Get DOS version
	int	21H			; Read vers into AX

	mov	bx,-1			; Set returned value
	cmp	al,3			; Version must be > 3
	jge	OPEN100 		; Skip if version OK

	xor	bx,bx			; bx -> 0
;
;	Store the DOS version check return value
;
OPEN100:
	pop	BP			; Restore saved BP
	mov	DI,[BP+4]		; Get offset of last parm
	mov	ax,[BP+6]		; Get segment of last parm
	mov	ES,ax			; And set into ES...

	mov	word ptr ES:[DI],bx	; Store returned value
;
;	Stop here if DOS version doesn't permit
;
	or	bx,bx			; Test if version does not allow us
	jz	OPENXIT 		; Exit now, if so
;
;	Set our own data segment
;
	mov	ax,Data 		; Setup our Dseg
	mov	DS,ax			; ..
;
;	Reset the flag we use to tell if a critical error 'hit'
;
	mov	Vector_Used,0		; Clear flag
;
;	If we haven't already taken the vector, take it now
;
	test	Saved_Vector,-1 	; Check high word of vector we save
	jnz	OPENXIT 		; Skip if it has already been taken
;
;	Read the current critical error vector, and save
;
	mov	al,24H			; Parm = 24 (int we're replacing)
	mov	ah,35H			; Ftn -> Get vector
	int	21H			; # in al, rtn ES:bx as ISR addr

	mov	Saved_Vector,bx 	; Store original value offset
	mov	Saved_Vector+2,ES	; Store original value offset
;
;	Establish our procedure as the new critical error vector
;
	push	CS			; Get Code space segment addr
	pop	DS			; Restore as DS for DOS call
	lea	dx,CS:Critical_Error	; Get address of procedure
	mov	al,24H			; Parm = 24 (int we're replacing)
	mov	ah,25H			; Ftn -> Set vector
	int	21H			; # in al, DS:Dx new ISR addr
;
;	And we're done
;
OPENXIT:
	ret				; FAR return here
Open	endp
page
;------ Close ------------------------------------------------------
;	This procedure 'closes' the module.  It restores the original
;	critical error vector (which in normal usage would be COM-AND)
;
;	Passed:
;	   Nothing
;	Returned:
;	   nothing
;
;	No registers need be preserved...
;-------------------------------------------------------------------

Close	proc	far			; MUST be far return !!
	mov	ax,Data 		; Setup our Dseg
	mov	DS,ax			; ..
;
;	If we haven't already taken the vector, we can't restore it
;
	test	Saved_Vector,-1 	; Check high word of vector we save
	jz	CLOS100 		; Skip if it has not been taken
;
;	Restore the vector from the saved value
;
	mov	dx,Saved_Vector 	; Get address of procedure
	mov	ax,Saved_Vector+2	; Get segment of procedure
	mov	DS,ax			; Set segment to passing reg
	mov	al,24H			; Parm = 24 (int we're replacing)
	mov	ah,25H			; Ftn -> Set vector
	int	21H			; # in al, DS:Dx new ISR addr
;
;	Reset the saved vector cell (so open can be called again)
;
CLOS100:
	mov	ax,Data 		; Setup our Dseg
	mov	DS,ax			; ..
	mov	Saved_Vector,0		; Clear cell
	mov	Saved_Vector+2,0	; ..
;
;	And we're done
;
	ret				; FAR return here
Close	endp
page
;------ GetFlag ----------------------------------------------------
;	This routine reads the flag used to record a call to the
;	critical error procedure.
;
;	Passed:
;	   nothing
;	Returned:
;	   One parameter, 0 -> Critical error vector not used
;
;	No registers need be preserved...
;-------------------------------------------------------------------

GetFlag proc	far			; MUST be far return !!
;
;	Get rtn parameter address
;
	mov	BP,SP			; Save current stack ptr

	mov	DI,[BP+4]		; Get offset of last parm
	mov	ax,[BP+6]		; Get segment of last parm
	mov	ES,ax			; And set into ES...
;
;	Set our segment
;
	mov	ax,Data 		; Setup our Dseg
	mov	DS,ax			; ..
;
;	Store the segment reg
;
	mov	ax,Vector_Used		; Read flag value
	mov	word ptr ES:[DI],ax	; Store it in caller's space
;
;	And we're done
;
	ret				; FAR return here
GetFlag endp
page
subttl Critical_Error: Critical error handler
;------ Critical_Error ------------------------------------------------
;	This procedure receives the DOS critical error vector.	Instead
;	of asking the usual: 1) Abort, 2) Retry, and 3) Ignore... this
;	procedure simply flags the invocation and returns 'fail' to DOS.
;
;	---------------------------------------------------------------
;	Note: This procedure, and any procedure called from it MAY NOT
;	invoke DOS (with very few exceptions). Calling DOS here causes it
;	to lose its stack... and thus locking up.
;	---------------------------------------------------------------
;
;	Entry:
;	  AX: Bit 7 -> clear if disk i/o, Set otherwise
;	  BP:SI -----> Device header control block
;	  DI lower byte -> error number (upper byte undefined)
;	Exit:
;	  Note: This procedure performs an IRET to the DOS with AL reg
;		set to reflect the choice made from the list above.
;	All registers are preserved.
;-----------------------------------------------------------------------

Critical_Error proc near
	push	bx			; Save register
	push	cx			; Save register
	push	dx			; Save register
	push	BP			; Save register
	push	SI			; Save register
	push	DI			; Save register
	push	ES			; Save register
	push	DS			; Save register
	push	ax			; Note: Save this reg last
;
;	Set-up our data segment
;
	mov	ax,Data 		; Setup our Dseg
	mov	DS,ax			; ..
;
;	Mark that the handler has been invoked
;
	mov	Vector_Used,-1		; Flag the fact
;
;	Return 'fail' (only good under DOS 3 and above
;
	pop	ax			; Restore passed ah value
	mov	al,3			; Return fail code in al
;
;	And we're done (note - saved AX is already popped)
;
	pop	DS			; Restore registers
	pop	ES			; Restore registers
	pop	DI			; Restore registers
	pop	SI			; Restore registers
	pop	BP			; Restore registers
	pop	dx			; Restore registers
	pop	cx			; Restore registers
	pop	bx			; Restore registers
	iret				; Return to DOS
Critical_Error endp
Code	ends
	end
