//**************************************************************************
//**
//**	Here is the code necessary to make a tap page. Since
//**    your serial functions will be different, they are not
//**    included. Even though many additional functions are not
//**    included, their names are descriptive enough to tell
//**    you what is actually going on inside the routines. The
//**    single biggest problem with sending an alpha message is
//**    making sure you handle all the modem and line problems.
//**    The PeopleFinder is a local box which removes 99.9% of
//**    all those problems.
//**
//**************************************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <share.h>
#include <io.h>
#include <dir.h>
#include <time.h>
#include <alloc.h>
#include <bios.h>

#include "serial.h"

#define NOERROR         0       // No error
#define BUFOVFL         1       // Buffer overflowed
#define SBUFSIZ         0x200	// Serial buffer size 512 bytes

#define SOH     		1
#define STX     		2
#define ETX     		3
#define EOT     		4
#define ACK     		6
#define NAK     		21
#define CAN				30
#define US				31

#ifndef ESC
 #define ESC			27
#endif

int         AlphaPage(int baud,char *tapnum,char *pin,char *passwd,char *msg);
void        CloseSerialChannel(void);
int         ComPortCheck(int ComDev);
int         CommWriteS(char *s,int flag);
int         ComputeCheckSum(char *s,int len);
int         DialModem(char *phono,int baud);
int 		DialPhone(char *ph);
int         FlushCommBuffers(void);
int         GetCommString(char *str,int sec,int len);
int         InKey(void);
int         LFStrip(char *str);
int         LeadingZeroTrim(char *str);
int         ModemCD(void);
int         ModemStatus(void);
void        OpenSerialChannel(int port);
int         SerialInput(void);
int         SerialInputStatus(void);
int         SerialOutput(char x);
void        SerialString(char *string);
void        SerialStringCR(char *string);
int         SetDTR(int flag);
int         SetOthers(int Parity, int Bits, int StopBit);
int         SetPort(int Port);
int         SetSerial(int Port, int Speed, int Parity, int Bits, int StopBit);
int         SetSpeed(int Speed);
int         StrPack(char *str);
int         StringMatch(char *str1, char *str2);
int         WaitKey(void);
int         WaitOn(int ch1,int ch2,int ch3,int sec);
int         WaitOnString(int timeout,char *s1,char *s2,char *s3,char *s4,char *s5,char *s6, char *s7, char *s8, char *s9);
int         check(void);
void        comm_off(void);
void        comm_on(void);
void        i_disable(void);
void        i_enable(int pnum);
static      int KeyTrans(int key);
void        interrupt com_int(void);
int         main(int argc, char *argv[]);
void        resvects(void);
void        setvects(void);

int             CommActive      = 0;
char            InString[256];
char            PinBuff[20];
char            MsgBuff[256];
int             SError          = NOERROR;
int             baud;
static   char   ccbuf[SBUFSIZ];
unsigned int    endbuf          = 0;
void            interrupt(*oldvects[2])();
int             port;                   // comm port for modem
int             portbase        = 0;
unsigned int    startbuf        = 0;

//**************************************************************************
//**
//**	Command line example:
//**
//**	TAP COMx 1200 TAP# PIN# PW# "THIS IS A TEST MESSAGE"
//**
//**
//**
//**************************************************************************
int main(int argc, char *argv[])
	{
    int ret;
    FILE *fs;

	if (argc != 7)
		{
		printf("Error in command line:\nTAP COMx 1200 TAP# PIN# PW# \"THIS IS A TEST MESSAGE\"\n\n");
		printf("Parm1 = Com Port\t\tCOM1 / COM2 / COM3 / COM4\n");
		printf("Parm2 = Baud Rate\t\t300  / 1200 / 2400\n");
		printf("Parm3 = TAP gateway telephone number\n");
		printf("Parm4 = Pager ID Number or ?\n");
		printf("Parm5 = TAP Password (usually 000000)\n");
		printf("Parm6 = \"quoted text message.\" or ?\n");
		return(1);
		}

	if (stricmp(argv[1],"COM1") == 0)		// Com1 selected?
		{
        port = 1;
		}
	else if (stricmp(argv[1],"COM2") == 0)
        {
        port = 2;
        }
	else if (stricmp(argv[1],"COM3") == 0)
        {
        port = 3;
        }
	else if (stricmp(argv[1],"COM4") == 0)
        {
        port = 4;
        }
    else
        {
        printf("Error: Invalid com port specified! %s\n",argv[1]);
        return(2);
        }

	if (!ComPortCheck(port))            	// validate com port
		{
		printf("COM%d does not exist!\n",port);
		return(3);
		}

	if (stricmp(argv[2],"300") == 0)       // 300 baud selected?
		{
        baud = 300;
		}
	else if (stricmp(argv[2],"1200") == 0) // 1200 baud selected?
        {
		baud = 1200;
        }
	else if (stricmp(argv[2],"2400") == 0) // 2400 baud selected?
        {
        baud = 2400;
        }
    else
        {
        printf("Error: Invalid baud rate specified! %s\n",argv[1]);
        return(4);
        }

	if (atol(argv[3]) < 1)                  // blank TAP number?
        {
        printf("Error: Invalid TAP gateway number specified! %s\n",argv[3]);
        return(4);
        }

    strcpy(PinBuff,argv[4]);                // Fill in the blank?
	if (PinBuff[0] == '?')
		{
        printf("Enter the PIN number: ");	// get pin number
        gets(PinBuff);
		}

	if (atol(PinBuff) < 1)
        {
        printf("Error: Invalid Pager ID number specified! %s\n",PinBuff);
        return(4);
        }

	strcpy(MsgBuff,argv[6]);				// fill in the blank message?
	if (MsgBuff[0] == '?')
		{
        printf("Enter the message: ");		// get message
        gets(MsgBuff);
		}

	if (MsgBuff[0] == '@')					// file input?
		{
		fs = fopen(&MsgBuff[1],"r");		// open input file
		if (fs)								// verify open
			{
        	fgets(MsgBuff,250,fs);			// get a line from the file
    	    StrPack(MsgBuff);				// remove unnecessary spaces
			fclose(fs);						// close file
			}
		else
			{
			printf("Error reading input file: %s\n",MsgBuff);
			return(5);
			}
		}

	if (StrPack(MsgBuff) < 1)				// check for blank message
		{
		printf("Cannot send a blank message!\n");
		return(4);
		}

	OpenSerialChannel(port);				// set up com handlers

	if (getenv("TAPINIT"))
		{
		CommWriteS(getenv("TAPINIT"),0);
		CommWriteS("!",0);
		}

	ret = AlphaPage(baud,argv[3],PinBuff,argv[5],MsgBuff);

	CloseSerialChannel();					// set up com handlers
	return(ret);
	}

//***************************************************************************
//**
//**	TAP is a packetized protocol to prevent transmission errors. The
//**	following code will generate the checksum of the passed string and
//**	return it to the calling function.
//**
//**    add all of the characters together and take lowest 12 bits of result
//**    convert result to ascii (3 digits) before transmitting to receiver.
//**
//***************************************************************************
int ComputeCheckSum(char *s,int len)
    {
    int i,j;

    j = 0;
    for (i=0;i<len;i++)
        j += (s[i] & 0x7F);
    return((j & 0xfff));
    }

//***************************************************************************
//**
//**	This function assumes that the com channel is valid and already open
//**	non-wonderus things can occur otherwise.
//**
//**	Use specified service definition to send alpha page.
//**	Message text is located in char var called msgbuf and must be less
//**	than the provider's maximum message length.
//**
//**	Always log outgoing pages to log file.
//**
//**	If any real C coders review this code, the answer is YES, it could
//**	have been written without a goto but it	is seldom needed.
//**
//**	baud		modem baud rate of alpha provider
//**	tapnum		phone number of alpha TAP gateway
//**	pin			Pager ID Number or alpha phone number
//**	msg			string to be sent to pager
//**	passwd		paging service provider password usually "0000000"
//**
//**	Function will return 1 if the page was accepted by the provider.
//**
//**
//***************************************************************************
int AlphaPage(int baud,char *tapnum,char *pin,char *passwd,char *message)
	{
	char	temp[260];
	int		RetryCount;
	int     i,l,done;

	if (atol(tapnum) == 0)
	    {
		printf("Invalid alpha phone number in service definition!\n");
		return(-1);
	    }

	if (atol(pin) == 0)
		{
		printf("Selected user does not have an alpha phone number defined.\n");
	  	return(-1);
		}

	printf("Dialing TAP host computer.\n");
	if (DialModem(tapnum,baud))						// dial modem and wait on CD
		return(-1);

	i = done = 0;
	while (i++ < 6 && !done)                        // make no more than 4 attempts.
        {
		printf("Sending Handshake!\n");
	    CommWriteS("\r",3);                         // wake up paging terminal
		if (WaitOn('=','I','I',2) != -1)   			// wait on password prompt
	        done = i = 10;                          // if we get a response...
	    }

	if (!done)
		{
	    printf("Host did not respond to handshake!\n");
	    return(-1);
		}

    printf("Sending system password.\n");
	sprintf(temp,"%cPG1%s!",ESC,passwd);			// send password
    CommWriteS(temp,0);
	if (WaitOn(0,ACK,NAK,15) != ACK)				// wait on pager id prompt
	    {
		printf("Host did not ACK login password!\n");
		return(-1);
		}
	RetryCount = 0;									// clear retry counter

TAP_Retry:
	if (ModemCD())				// make sure modem carrier is still up
		{
		check();
		sprintf(temp,"%c%s\r%s\r%c",STX,pin,message,ETX);
		printf("Sending pager id number & message.\n");
		CommWriteS(temp,1);                 	// send pager id number
		i = ComputeCheckSum(temp,strlen(temp));
		temp[0] = ((i >> 8) & 0x0f) + 0x30;		// make a string of checksum
		temp[1] = ((i >> 4) & 0x0f) + 0x30;
		temp[2] = (i & 0x0f) + 0x30;
		temp[3] = '\r';							// terminate checksum with CR
		temp[4] = '\0';
		printf("Sending checksum to host.\n");
		CommWriteS(temp,0); 					// send packet checksum string
		switch (WaitOn(NAK,ACK,CAN,6))     		// wait on message ack
			{

			case ACK:
				printf("Alpha Page Sent Properly.\n");
				break;

			case NAK:
				printf("NAK received from host!\n");
				if (RetryCount)
					{
					printf("Attempting a second transmission\n");
					goto TAP_Retry;
					}
	            return(-1);

			case CAN:
				printf("CAN received from host!\n");
	            return(-1);

			case -1:
				printf("No valid response from host!\n");
	            return(-1);

			default:
				printf("Unknown response from host!\n");
	            return(-1);

			}
		}

	if (ModemCD())
		CommWriteS("!",0);         		// terminate connection
	if (WaitOn(EOT,0,0,10) != EOT)    		// wait on EOT
		printf("No EOT received from host!\n");
	check();
	delay(1000);
	return(1);
    }

//***************************************************************************
//**
//**	eat any incoming serial data on the serial port.
//**
//***************************************************************************
int check(void)
    {
    int i;

    while (SerialInputStatus())
        {
        while (SerialInputStatus())
			SerialInput();
        }
    return(0);
    }

//***************************************************************************
//**
//**	Wait on remote to send either ch1 or ch2 or ch3. return with char
//**    whichwas received to cause the release. if none of the allowed chars
//**    are not received within sec seconds return zero.
//**
//***************************************************************************
int WaitOn(int ch1,int ch2,int ch3,int sec)
    {
    int     i;
    long	t,to;
	int		off = 0;

	memset(InString,'\0',sizeof(InString));
	to = time(&t) + (long) sec;		// get time now and inc timeout

	while (time(NULL) < to)			// loop until found or timeout
		{
		i = SerialInput();
		if (i == ch1 || i == ch2 || i == ch3)	// if it is one of the chars
			{
			return(i);
			}
		if (i > 31 && i < 123 &&
			off < sizeof(InString))			// log input string
			InString[off++] = i;			// add char to string
 		}
	return(-1);
	}

//***************************************************************************
//**
//**	Return modem Carrier Detect status
//**
//***************************************************************************
int ModemCD(void)
	{
	delay(250);
	return(ModemStatus() & DCD);
	}

//***************************************************************************
//**
//**	Handle communications interrupts and put them in ccbuf
//**
//***************************************************************************
void interrupt com_int(void)
	{

    disable();
    if ((inportb(portbase + IIR) & RX_MASK) == RX_ID)
	    {
        if (((endbuf + 1) & SBUFSIZ - 1) == startbuf)
            SError = BUFOVFL;

        ccbuf[endbuf++] = inportb(portbase + RXR);
        endbuf &= SBUFSIZ - 1;
    	}

    /* Signal end of hardware interrupt */
    outportb(ICR, EOI);
    enable();
	}

//***************************************************************************
//**
//**	Output a character to the serial port
//**
//***************************************************************************
int SerialOutput(char x)
	{
    long int           timeout = 0x0000FFFFL;

//	printf(">%c\n",x);

    outportb(portbase + MCR,  MC_INT | DTR | RTS);

    /* Wait for Clear To Send from modem */
    while ((inportb(portbase + MSR) & CTS) == 0)
        if (!(--timeout))
            return (-1);

    timeout = 0x0000FFFFL;

    /* Wait for transmitter to clear */
    while ((inportb(portbase + LSR) & XMTRDY) == 0)
        if (!(--timeout))
            return (-1);

    disable();
    outportb(portbase + TXR, x);
    enable();

    return (0);
	}

//***************************************************************************
//**
//**	Output a string to the serial port
//**
//***************************************************************************
void SerialString(char *string)
	{
    while (*string)
    	{
       	SerialOutput(*string++);
       	delay(50);
       	}
	}

//***************************************************************************
//**
//**	Output a string to the serial port followed by CR
//**
//***************************************************************************
void SerialStringCR(char *string)
	{
    while (*string)
    	{
       	SerialOutput(*string++);
       	delay(50);
       	}
    SerialOutput('\r');
	}

//***************************************************************************
//**
//**	This routine returns the current value in the buffer
//**
//***************************************************************************
int SerialInput(void)
	{
    int res;

    if (endbuf == startbuf)
        return (-1);

    res = (int) ccbuf[startbuf++];
    startbuf %= SBUFSIZ;

//	printf("<%c\n",res);

    return (res);
	}

//***************************************************************************
//**
//**	return 1 if there are chars in the input buffer
//**
//***************************************************************************
int SerialInputStatus(void)
	{
	return(endbuf != startbuf);
	}

//***************************************************************************
//**
//**	return current setting of modem control registers.
//**
//***************************************************************************
int ModemStatus(void)
	{
	return(inportb(portbase + MSR));
    }

//***************************************************************************
//**
//**	Install our int vectors to handle communications
//**
//***************************************************************************
void setvects(void)
	{

#if 0
    oldvects[0] = getvect(0x0B);
    oldvects[1] = getvect(0x0C);
    setvect(0x0B, com_int);
    setvect(0x0C, com_int);
#else
	switch (portbase)
        {

		case COM1BASE:
		    oldvects[0] = getvect(0x0C);
		    setvect(0x0C, com_int);
            break;

		case COM2BASE:
		    oldvects[0] = getvect(0x0B);
		    setvect(0x0B, com_int);
            break;

		case COM3BASE:
		    oldvects[0] = getvect(0x0C);
		    setvect(0x0C, com_int);
            break;

		case COM4BASE:
		    oldvects[0] = getvect(0x0B);
		    setvect(0x0B, com_int);
            break;
        }
#endif
	return;
	}

//***************************************************************************
//**
//**	Uninstall our vectors before exiting the program
//**
//***************************************************************************
void resvects(void)
	{

#if 0
    setvect(0x0B, oldvects[0]);
    setvect(0x0C, oldvects[1]);
#else
	switch (portbase)
        {

		case COM1BASE:
		    setvect(0x0C, oldvects[0]);
            break;

		case COM2BASE:
    		setvect(0x0B, oldvects[0]);
            break;

		case COM3BASE:
		    setvect(0x0C, oldvects[0]);
            break;

		case COM4BASE:
    		setvect(0x0B, oldvects[0]);
            break;
        }
#endif
	return;
	}

//***************************************************************************
//**
//**	Turn on communications interrupts
//**
//***************************************************************************
void i_enable(int pnum)
	{
    int c;

    disable();
    c = inportb(portbase + MCR) | MC_INT;
    outportb(portbase + MCR, c);
    outportb(portbase + IER, RX_INT);
    switch (pnum)
    	{
    	case COM1:
    	case COM3:
    		c = inportb(IMR) & IRQ4;
    		break;

    	case COM2:
    	case COM4:
    		c = inportb(IMR) & IRQ3;
    		break;

        }
    outportb(IMR, c);
    enable();
	}

//***************************************************************************
//**
//**	Turn off communications interrupts
//**
//***************************************************************************
void i_disable(void)
	{
    int c;

    disable();
	switch (portbase)
        {

		case COM1BASE:
		    c = inportb(IMR) | ~IRQ4;
		    outportb(IMR, c);
            break;

		case COM2BASE:
		    c = inportb(IMR) | ~IRQ3;
		    outportb(IMR, c);
            break;

		case COM3BASE:
		    c = inportb(IMR) | ~IRQ4;
		    outportb(IMR, c);
            break;

		case COM4BASE:
		    c = inportb(IMR) | ~IRQ3;
		    outportb(IMR, c);
            break;
        }
    outportb(portbase + IER, 0);
    c = inportb(portbase + MCR) & ~MC_INT;
    outportb(portbase + MCR, c);
    enable();
    return;
	}

//***************************************************************************
//**
//**	Tell modem that we're ready to go
//**
//**
//**
//***************************************************************************
void comm_on(void)
	{
    int c, pnum;

	switch (portbase)
        {

		case COM1BASE:
            pnum = COM1;
            break;

		case COM2BASE:
            pnum = COM2;
            break;

		case COM3BASE:
            pnum = COM3;
            break;

		case COM4BASE:
            pnum = COM4;
            break;
        }

    i_enable(pnum);
    c = inportb(portbase + MCR) | DTR | RTS;
    outportb(portbase + MCR, c);
	}

//***************************************************************************
//**
//**	Go off-line
//**
//***************************************************************************
void comm_off(void)
	{
    i_disable();
    outportb(portbase + MCR, 0);
	}

//***************************************************************************
//**
//**	Set up whatever is necessary for serial communications on the
//**	specifed com port.
//**
//***************************************************************************
void OpenSerialChannel(int port)
	{

	endbuf = startbuf = 0;
	SetPort(port);
    setvects();
	comm_on();
	CommActive = 1;
	if ((ModemStatus() & DSR) == 0)		// if DSR is off
        CommWriteS("AT&S0!",0);			// set it on!
	}

//***************************************************************************
//**
//**	Release the current com port vectors
//**
//***************************************************************************
void CloseSerialChannel(void)
	{

	CommActive = 0;
    comm_off();
	resvects();
	}

//***************************************************************************
//**
//**	This should hang up the modem (if flag is false)
//**
//***************************************************************************
int SetDTR(int flag)
    {
	int c;

    if (!flag)
 	    comm_off();
    else
    	{
    	comm_on();
	    delay(500);
    	}
    FlushCommBuffers();
    return(0);
    }

//***************************************************************************
//**
//**	Set the port number to use
//**
//***************************************************************************
int SetPort(int Port)
	{

    switch (Port)		// Sort out the base address
    	{

      	case COM1 :
      		portbase = COM1BASE;
            break;

	    case COM2 :
      		portbase = COM2BASE;
            break;

        case COM3:
        	portbase = COM3BASE;
        	break;

        case COM4:
        	portbase = COM4BASE;
        	break;

      	default :
      		return (-1);
	    }

    return (0);
	}

//***************************************************************************
//**
//**	This routine sets the speed; will accept funny baud rates.
//**    Setting the speed requires that the DLAB be set on.
//**
//**
//***************************************************************************
int SetSpeed(int Speed)
	{
    char		c;
    int		divisor;

    if (Speed == 0)            /* Avoid divide by zero */
        return (-1);
    else
        divisor = (int) (115200L/Speed);

    if (portbase == 0)
        return (-1);

    disable();
    c = inportb(portbase + LCR);
    outportb(portbase + LCR, (c | 0x80)); /* Set DLAB */
    outportb(portbase + DLL, (divisor & 0x00FF));
    outportb(portbase + DLH, ((divisor >> 8) & 0x00FF));
    outportb(portbase + LCR, c);          /* Reset DLAB */
    enable();

    return (0);
	}

//***************************************************************************
//**
//**	Set other communications parameters
//**
//***************************************************************************
int SetOthers(int Parity, int Bits, int StopBit)
	{
    int setting;

    if (portbase == 0)
    	return (-1);
    if (Bits < 5 || Bits > 8)
    	return (-1);
    if (StopBit != 1 && StopBit != 2)
    	return (-1);
    if (Parity != NO_PARITY && Parity != ODD_PARITY && Parity != EVEN_PARITY)
		return (-1);

    setting  = Bits-5;
    setting |= ((StopBit == 1) ? 0x00 : 0x04);
    setting |= Parity;

    disable();
    outportb(portbase + LCR, setting);
    enable();

    return (0);
	}

//***************************************************************************
//**
//**	Set up the port
//**
//***************************************************************************
int SetSerial(int Port, int Speed, int Parity, int Bits, int StopBit)
	{

    if (SetPort(Port))
    	return (-1);
    if (SetSpeed(Speed))
    	return (-1);
    if (SetOthers(Parity, Bits, StopBit))
    	return (-1);

    return (0);
	}

//***************************************************************************
//**
//**	Make the input buffer data go away...
//**
//**
//**
//***************************************************************************
int FlushCommBuffers(void)
	{

    endbuf = startbuf = 0;
    return(0);
    }

/**************************************************************************\
*                                                                          *
*   Check to see if there is a com port there... If not, they return FF    *
*   and some other number if one is there.                                 *
*                                                                          *
\**************************************************************************/
int ComPortCheck(int ComDev)
	{

	switch (ComDev)
		{
		case 0:
			return(0);

        case 1:
            if ((inportb(0x3F8+4)&0xE0))
				return(0);
			break;

        case 2:
            if ((inportb(0x2F8+4)&0xE0))
				return(0);
			break;

        case 3:
            if ((inportb(0x3E8+4)&0xE0))
				return(0);
			break;

        case 4:
            if ((inportb(0x2E8+4)&0xE0))
				return(0);
			break;

		default:
            if (inportb(ComDev+4)&0xE0)
				return(0);
            break;
        }
    return(1);
	}

//***************************************************************************
//**
//**    This function will scan incoming serial data for matching strings.
//**    it returns the string number of the matching string.
//**
//***************************************************************************
int WaitOnString(int timeout,char *s1,char *s2,char *s3,char *s4,char *s5,char *s6, char *s7, char *s8, char *s9)
	{
    long to;
    char temp[120];

	to = time(NULL) + timeout;			// compute timeout.
    while (time(NULL) < to)
    	{
        if (GetCommString(temp,12,80))  // get string from com port
			{

			if (StringMatch(s1,temp))   // is it string 1?
				return(1);

			if (StringMatch(s2,temp))   // is it string 2?
				return(2);

			if (StringMatch(s3,temp))   // is it string 3?
				return(3);

			if (StringMatch(s4,temp))   // is it string 4?
				return(4);

			if (StringMatch(s5,temp))    // is it string 5?
            	return(5);

			if (StringMatch(s6,temp))    // is it string 6?
            	return(6);

			if (StringMatch(s7,temp))    // is it string 7?
            	return(7);

			if (StringMatch(s8,temp))    // is it string 8?
            	return(8);

			if (StringMatch(s9,temp))    // is it string 9?
            	return(9);

            }
        }
    return(-1);
	}

//***************************************************************************
//**
//**	set up modem and dial phone number.
//**
//**	Modem commands are on separate lines to prevent errors from skipping
//**	correct commands.
//**
//**
//**
//**
//**
//**
//***************************************************************************
int DialPhone(char *ph)
	{

	CommWriteS("ATM1!",0);			// enable speaker
	CommWriteS("ATE1!",0);			// enable echo
	CommWriteS("ATX4!",0);			// use CPA
	CommWriteS("AT&C1&D2!",0);		// and tie DTS
	CommWriteS("ATDT",1);			// Send dial prefix
	CommWriteS(ph,0);				// and phone number
	CommWriteS("!",0);				// send it!
	return(0);
	}

//***************************************************************************
//**
//**	Function to dial modem and wait for a digital connection.
//**	Function returns zero if connection was made.
//**
//***************************************************************************
int DialModem(char *phono,int baud)
	{
	char	temp[85];
    int 	done;

	SetDTR(1);
	if (baud != 300 &&
		baud != 1200 &&
        baud != 2400 &&
        baud != 9600)
		SetSerial(port,300,EVEN_PARITY,7,1);
	else
		SetSerial(port,baud,EVEN_PARITY,7,1);

	DialPhone(phono);						// issue the dial command
	done = 5;

	while (done-- > 0)
		{

		if (InKey() == ESC)						// escape key?
			{
			printf("User Interrupt!!!\n");
			return(-1);
			}

		if (GetCommString(temp,45,80) == 0)     // get next line
			{
			printf("No response from modem after dial command!\n");
			return(-1);
    		}

		if (StringMatch("NO DIAL",temp))		// phone busy?
			{
			printf("No dialtone on modem phone line!\n");
			return(-1);
			}

		if (StringMatch("BUSY",temp))  			// phone busy?
			{
			printf("Host phone number is currently busy!\n");
			return(-1);
			}

		if (StringMatch("NO ANSWER",temp))  	// asking for user number?
			{
			printf("No answer when dialing host!\n");
			return(-1);
			}

		if (StringMatch("NO CARRIER",temp))  	// asking for user number?
			{
			printf("No Carrier reported from modem following dial command!\n");
			return(-1);
			}

		if (StringMatch("CONNECT",temp) ||		// if not connect string
			StringMatch("1",temp) ||			// or connect 300
			StringMatch("5",temp))				// or connect 1200
			{
			delay(1000);                        // wait a second
			if (!ModemCD())						// make sure modem carrier is up
				{
				printf("Modem reports Carrier Detect is off!\n");
				return(-1);
				}
	    	printf("On line with host computer.\n");
			return(0);
			}
		}
	printf("Unknown response from modem after dial command!\n");
	return(-1);
	}

//***************************************************************************
//**
//**	Output a string to port specified.
//**
//**    Stop once null char is received.
//**
//***************************************************************************
int CommWriteS(char *s,int flag)
    {
    int i=0;

    while (s[i])
        {
        if (flag)                   // if flag is true leave input alone
            SerialOutput(s[i]);     // send it out without any translation
        else
            {
            switch (s[i])
                {

                case '!' :          // Carriage return & delay
                    check();
                    SerialOutput(13);
                    delay(250);
                    break;

                case '~' :          // delay for 1/4th second
                    delay(250);
                    break;

                case '$' :          // delay for 1/4th second
                    SerialOutput(',');
                    break;

                case '^' :          // convert next character to CONTROL CHAR
                    i++;
                    SerialOutput(s[i]&0x1F);
                    break;

                default :           // just output the character
                    SerialOutput(s[i]);
                    break;
                }
			delay(50);
            }
        i++;
        }
    return(i);
    }

//***************************************************************************
//**
//**	Trim whitespace from either left or right side of string.
//**    Also remove any redundant spaces from within string.
//**
//**    procedure:
//**    convert any tabs to spaces
//**    remove CR/LF from end of string
//**    remove any spaces from end of string
//**    remove any spaced from beginning of string
//**    look for two or more spaces together in the string
//**
//***************************************************************************
int StrPack(char *str)
	{
    int l,i;

	l = strlen(str);			// get initial string length
	for (i=0;i<l;i++)			// translate tabs to spaces
		if (str[i] == '\t')		// if char is a tab, make it a space
			str[i] = ' ';

    l = strlen(str) - 1;
	while (l>0 && (str[l] == ' ' || str[l] == '\r' || str[l] == '\n'))
		str[l--] = '\0';

    l = strlen(str);
	while ((str[0] == ' ' || str[0] == '\t') && l)
        memmove(&str[0],&str[1],l--);

    l = strlen(str);
    i = 0;
	while (str[i+1] != '\0')
		{
		if (str[i] == ' ' && str[i+1] == ' ')
			{
	        memmove(&str[i],&str[i+1],l-i);
	        l--;
	        }
	    else
	    	i++;
    	}

	return(LFStrip(str));
	}

//***************************************************************************
//**
//**	Strip off any linefeeds at the end of the string.
//**
//***************************************************************************
int LFStrip(char *str)
	{
    int l;

	l = strlen(str) - 1;					// get string length
	while (str[l] == 10 || str[l] == 13)	// if last line is CR or LF
		str[l--] = '\0';					// truncate it

    return(strlen(str));					// return new length
	}

//***************************************************************************
//**
//**	function to see if two strings match.
//**
//***************************************************************************
int StringMatch(char *str1, char *str2)
    {
	int l = strlen(str1);

	if (strncmpi(str1,str2,l) == 0)     // parse user pages
		return(1);
	return(0);
    }

//***************************************************************************
//**
//**	Function to build a string from the comm channel.
//**
//**	Return value is number of bytes in string.
//**
//***************************************************************************
int GetCommString(char *str,int sec,int len)
	{
    int i,cc;
    long to;

	cc = 0;
	str[cc] = '\0';
    to = time(NULL) + (long) sec;
	while (time(NULL) < to && cc <= len)
		{
        if (SerialInputStatus())
        	{
			i = SerialInput();			// get next serial char
			if (i == 10 || i == 13)		// EOS?
				{
				if (strlen(str))
					{
	        	    return(strlen(str));
	            	}
		        }
		    else
	    		{
				if (i > 0 || i < 256)
					{
	        	    str[cc++] = i;
					str[cc] = '\0';
					}
				}
			}
		}
	return(strlen(str));
	}

//***************************************************************************
//**
//**	Remove any leading zeros from number.
//**
//***************************************************************************
int LeadingZeroTrim(char *str)
	{

    while (str[0] == '0')
        memmove(&str[0],&str[1],strlen(str));
	return(strlen(str));
	}

//**************************************************************************
//**
//**	Function to translate keyboard input into standard keycodes.
//**
//**************************************************************************
static int KeyTrans(int key)
	{

    if (key & 0xff)			// is it a special key?
        return(key & 0xff);	// if not, return it

	return((key>>8)+256);	//
	}

//**************************************************************************
//**
//**	If a keystroke is buffered, return it to caller, otherwise return 0
//**
//**************************************************************************
int InKey(void)
    {
    int key;

	key = bioskey(1);			// if a key exists, input it.
	if (key)
		{
		key = bioskey(0);		// input it.
		key = KeyTrans(key);	// translate keyboard input
		}
	return(key);
	}

//**************************************************************************
//**
//**	Wait on a keystroke, translate it and return keycode to caller.
//**
//**************************************************************************
int WaitKey(void)
	{
	int key;

	key = InKey();				// get keystatus
	while (!key)				// loop until a key is ready
		key = InKey();			// get new status
	return(key);				// return key
	}

